/*********************************************************************
 *		ArtJoystick Firmware, based heavily on
 *		Microchip USB C18 Firmware Version 1.0
 *		Rawin Rojvanit       19/11/04    Original.
 *
 *		Note: Though original code is excellent Microchip PIC18Fx550
 *		USB firmware framework, it is too complex for me to take fun
 *		with, so I reshaped it.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *	-	Artem				 06/04/06	 Joystick modification, 0.01
 *	-						 12/04/06	 Revision 0.03 -- USB/HID
 *										 complete
 *	-
 *
 *********************************************************************/

//
#include <string.h>
#include <delays.h>

//
#include "AJo.h"


// MCU configuration
// --------------------------------------------------------------------------------------
#pragma config LVP = ON, WDT = OFF, IESO = OFF, VREGEN = ON, PBADEN = OFF
#pragma config PLLDIV = 1, CPUDIV = OSC1_PLL2, USBDIV = 2, FOSC = HSPLL_HS

// Descriptors
// --------------------------------------------------------------------------------------
#pragma romdata

// Device Descriptor
rom DeviceDescriptorFormat DeviceDescriptor =
{    
    sizeof(DeviceDescriptorFormat),		// Size of this descriptor in bytes
    DEVICE,								// DEVICE descriptor type
//	0x0200,								// 2.0 USB Spec Release Number in BCD format
	0x0110,								// 1.1
    0x00,								// Class Code
    0x00,								// Subclass code
    0x00,								// Protocol code
    EP0_BufferSize,						// Max packet size for EP0, see usbcfg.h
    0x0000,								// Vendor ID: 0x0000 reserved for development
    0x0001,								// Product ID:
    0x0003,								// Device release number in BCD format
    0x01,								// Manufacturer string index
    0x02,								// Product string index
    0x00,								// Device serial number string index
    0x01								// Number of possible configurations
};


// Report descriptors
// --------------------------------------------------------------------------------------
// Joystick 00
#define REPORT_SIZE_00			50
rom byte ReportDescriptor_00[REPORT_SIZE_00] =
{
    0x05, 0x01,			// Usage Page (Generic Desktop)
    0x09, 0x04,			// Usage (Joystick)
    0xA1, 0x01,			// Collection (Application)
    0x09, 0x01,			//  Usage (Pointer)
    0xA1, 0x00,			//  Collection (Physical)
    0x05, 0x01,			//      Usage Page (Generic Desktop)
    0x09, 0x30,			//      Usage (X)
    0x09, 0x31,			//      Usage (Y)
    0x15, 0x81,			//      Logical Minimum (-127)
    0x25, 0x7F,			//      Logical Maximum (127)
    0x75, 0x08,			//      Report Size (10)
    0x95, 0x02,			//      Report Count (2)
    0x81, 0x02,			//      Input (Data, Variable, Relative)
    0x05, 0x09,			//      Usage Page (Buttons)
    0x19, 0x01,			//      Usage Minimum (01)
    0x29, 0x03,			//      Usage Maximum (03)
    0x15, 0x00,			//      Logical Minimum (0)
    0x25, 0x01,			//      Logical Maximum (0)
    0x95, 0x03,			//      Report Count (3)
    0x75, 0x01,			//      Report Size (1)
    0x81, 0x02,			//      Input (Data, Variable, Absolute)
    0x95, 0x01,			//      Report Count (1)
    0x75, 0x05,			//      Report Size (5)
    0x81, 0x01,			//      Input (Constant) - 5 bit padding
    0xC0, 0xC0			// End Collection,End Collection
};

// Joystick 01
#define REPORT_SIZE_01			74
rom byte ReportDescriptor_01[REPORT_SIZE_01] =
{
    0x05, 0x01,			// Usage Page (Generic Desktop)
    0x09, 0x04,			// Usage (Joystick)
    0xA1, 0x01,			// Collection (Application)
    0x09, 0x01,			//  Usage (Pointer)
    0xA1, 0x00,			//  Collection (Physical)					; 10

    0x05, 0x01,			//      Usage Page (Generic Desktop)
    0x09, 0x30,			//      Usage (X)
    0x16, 0x01,	0xFE,	//      Logical Minimum
    0x26, 0xFF,	0x01,	//      Logical Maximum
    0x75, 0x0A,			//      Report Size
    0x95, 0x01,			//      Report Count
    0x81, 0x02,			//      Input (Data, Variable, Absolute)	; 26

    0x05, 0x01,			//      Usage Page (Generic Desktop)
    0x09, 0x31,			//      Usage (Y)
    0x15, 0x00,			//      Logical Minimum
    0x26, 0xFF,	0x03,	//      Logical Maximum
    0x75, 0x0A,			//      Report Size
    0x95, 0x01,			//      Report Count
    0x81, 0x02,			//      Input (Data, Variable, Absolute)	; 41

    0x05, 0x01,			//      Usage Page (Generic Desktop)
    0x09, 0x32,			//      Usage (Z)
    0x15, 0x00,			//      Logical Minimum
    0x26, 0xFF,	0x03,	//      Logical Maximum
    0x75, 0x0A,			//      Report Size
    0x95, 0x01,			//      Report Count
    0x81, 0x02,			//      Input (Data, Variable, Absolute)	; 56
/*
    0x05, 0x01,			//      Usage Page (Generic Desktop)
    0x09, 0x33,			//      Usage (Rx)
    0x15, 0x00,			//      Logical Minimum
    0x26, 0xFF,	0x03,	//      Logical Maximum
    0x75, 0x0A,			//      Report Size
    0x95, 0x01,			//      Report Count
    0x81, 0x02,			//      Input (Data, Variable, Absolute)	;
*/
    0x05, 0x09,			//      Usage Page (Buttons)
    0x19, 0x01,			//      Usage Minimum
    0x29, 0x08,			//      Usage Maximum
    0x15, 0x00,			//      Logical Minimum
    0x25, 0x01,			//      Logical Maximum
    0x95, 0x02,			//      Report Count
    0x75, 0x01,			//      Report Size
    0x81, 0x02,			//      Input (Data, Variable, Absolute)

//    0x95, 0x01,			//      Report Count
//    0x75, 0x04,			//      Report Size
//    0x81, 0x01,			//      Input (Constant) - bit padding
    0xC0, 0xC0			// End Collection,End Collection			;
};

// HID Descriptor
// --------------------------------------------------------------------------------------
rom HIDFormat1 HIDDescriptor1 =
{
	sizeof(HIDFormat1),
	CLASS_HID,
	0x0101,										// HID Spec Release Number in BCD format
	0x00,										// Country code (0x00 for "not supported")
	0x02,										// Description number to request
	CLASS_REPORT,								// Descriptor type to request
	REPORT_SIZE_01,								// Size to request, word
	CLASS_HID,									// Descriptor type to request
	sizeof(HIDFormat1),							// Size to request, word
};

// Configuration 1 Descriptor
// --------------------------------------------------------------------------------------
rom USBConfigurationStruct ConfigurationDescriptor =
{
    // Configuration Descriptor
    sizeof(DeviceConfigurationFormat),    		// Size of this descriptor in bytes
    CONFIGURATION,                				// CONFIGURATION descriptor type
    sizeof(USBConfigurationStruct),				// Total length of data for this cfg
												// sizeof(DeviceConfigurationFormat) + sizeof(DeviceInterfaceFormat) + ...
    0x01,                      					// Number of interfaces in this cfg
    0x01,                      					// Index value of this configuration
    0x00,                      					// Configuration string index
    _DEFAULT,		          					// Attributes, see usbdefs_std_dsc.h
    50,                     					// Max power consumption (2X mA)

	// Interface descriptor
	sizeof(DeviceInterfaceFormat),				// Size of this descriptor in bytes
	INTERFACE,									// INTERFACE descriptor type
	0x00,										// Interface number (zero based)
	0x00,										// Alternate setting
	0x01,										// Number of endpoints
	0x03,										// Interface class (0x3 is HID)
//	0x00,
	0x00,										// Interface subclass
	0x00,										// Protocol
	0x04,										// Interface Sting index

	// HID -- Just have to repeat initialisation here, cose scilless in "C" :-)
	sizeof(HIDFormat),
	CLASS_HID,
	0x0101,										// HID Spec Release Number in BCD format
	0x00,										// Country code (0x00 for "not supported")
	0x01,										// Number of descriptors to request
	CLASS_REPORT,								// Descriptor type to request
	REPORT_SIZE_01,								// Size to request, word
//	CLASS_HID,									// Descriptor type to request
//	sizeof(HIDFormat1),							// Size to request, word

	// Endpoint
	sizeof(DeviceInterfaceFormat),				// Size of this descriptor in bytes
	ENDPOINT,									// ENDPOINT descriptor type
	0b10000001,									// Endpoint address
	0b00000011,									// Attributes
	0x08,										// Max packet size
	6											// Polling interval, ms
};


// Strings
// --------------------------------------------------------------------------------------
rom struct{byte bLength; byte bDscType; word string[2];} LangIDs = { sizeof(LangIDs), STRING, 0x0409, 0x419 };		// 0

rom struct{byte bLength; byte bDscType; word string[5];} ArtemStr = { sizeof(ArtemStr), STRING,						// 1
'A','r','t','e','m' };
rom struct{byte bLength; byte bDscType; word string[22];} ProductStr = { sizeof(ProductStr), STRING,				// 2
'A','r','t','J','o','y','s','t','i','c','k',',',' ','r','e','v','.',' ','0','.','0','3' };
rom struct{byte bLength; byte bDscType; word string[4];} SerialStr = { sizeof(SerialStr), STRING,					// 3
'0','0','0','1' };
rom struct{byte bLength; byte bDscType; word string[15];} InterfaceStr = { sizeof(InterfaceStr), STRING,			// 4
'A','r','t','J','o','y','s','t','i','c','k',' ','-',' ','1' };


//	U S B  F I X E D  L O C A T I O N  V A R I A B L E S
// --------------------------------------------------------------------------------------
#pragma udata usbram4=0x400     			//See Linker Script,usb4:0x400-0x4FF(256-byte)

volatile far BDTable EP0_Out;				//Endpoint #0 BD Out
volatile far BDTable EP0_In;				//Endpoint #0 BD In
volatile far BDTable EP1_Out;
volatile far BDTable EP1_In;


volatile far SetupDataFormat SetupData;
volatile far byte EP0_Data[8];

volatile far byte EP1_OutData[EP1_BufferSize];
// volatile far byte EP1_InData[EP1_BufferSize];
volatile far ReportFormat01 Report01;


#pragma udata
// Global variables
// --------------------------------------------------------------------------------------
// Log
byte LogIndex = 0;
byte Log[LOGSIZE];

byte Stopped = 0;

// ADC
WORD ADCSense[ChannelsNum];


// File variables
// --------------------------------------------------------------------------------------
static byte USBDeviceState = DETACHED_STATE;				// Device States: DETACHED, ATTACHED, ...
static byte Need2Stop = 0;


#define TransferBlinkDelay		300
//
// --------------------------------------------------------------------------------------
void ServeEP1(USBStatusReg USBStatus)
{
	static unsigned short int GreenCount = TransferBlinkDelay;
	static int j = 0;
	byte i = 0;


	RedLed(Off);
	YellowLed(Off);
	GreenLed(Free);

	if(GreenCount++ >= TransferBlinkDelay)
	{
		GreenCount = 0;
		PORTBbits.RB2 = ~LATBbits.LATB2;
	}

	//
	ADCSense[0]._word -= 512;
	Report01.XL = ADCSense[0].byte0;
	Report01.XH = ADCSense[0].byte1;

//	ADCSense[1]._word -= 512;
	Report01.YL = ADCSense[1].byte0;
	ADCSense[1]._word <<= 2;
	Report01.YH = ADCSense[1].byte1;

//	ADCSense[2]._word -= 512;
	Report01.ZL = ADCSense[2].byte0;
	ADCSense[2]._word <<= 4;
	Report01.ZH = ADCSense[2].byte1;


	if(EP1_In.Status.DTS == 0)				// Toggle DTS bit and turn ownership to SIE
		EP1_In.Status._byte = _USIE | _DAT1 | _DTSEN;
	else
		EP1_In.Status._byte = _USIE | _DAT0 | _DTSEN;

//	Stopped = 1;		// For streaming log 2 eeprom after full setup
}

//
// --------------------------------------------------------------------------------------
void ServeEP0(USBStatusReg USBStatus)
{
	static rom byte *DataPointer = 0;
	static byte DataCount = 0;
	static byte USBAddress = 0;

	if(USBStatus.Dir == OUT)
	{
		switch(EP0_Out.Status.PID)
		{
			case SETUP_TOKEN:
				if(USBDeviceState >= ADDRESSED_STATE)
				{
					byte2log(0xAA);
					ram2log((byte *)&SetupData, EP0_BufferSize);
				}

				switch(SetupData.bmRequestType)
				{
					case bmGET_xxxx:
						RedLed(Off);		// 0x0010
						YellowLed(On);
						GreenLed(Off);

						switch(SetupData.bRequest)
						{
							case bGET_DESCRIPTOR:
								switch(SetupData.DescriptorType)
								{
									case DEVICE:
										memcpypgm2ram(EP0_Data, (rom byte *)&DeviceDescriptor, EP0_BufferSize);
										DataPointer = (rom byte *)&DeviceDescriptor;
										DataCount = sizeof(DeviceDescriptorFormat);
									break;

									case CONFIGURATION:
										memcpypgm2ram(EP0_Data, (rom byte *)&ConfigurationDescriptor, EP0_BufferSize);
										DataPointer = (rom byte *)&ConfigurationDescriptor;
										if(SetupData.DataSize >= sizeof(USBConfigurationStruct))
											DataCount = sizeof(USBConfigurationStruct);
										else
											DataCount = SetupData.DataSize;
									break;

									case SOME01:
										memcpypgm2ram(EP0_Data, (rom byte *)&ConfigurationDescriptor, EP0_BufferSize);
										DataPointer = (rom byte *)&ConfigurationDescriptor;
										DataCount = 0x0A;
									break;

									case STRING:
										switch(SetupData.bIndex)
										{
											case 0:
												memcpypgm2ram(EP0_Data, (rom byte *)&LangIDs, sizeof(LangIDs));
												DataPointer = (rom byte *)&LangIDs;
												DataCount = sizeof(LangIDs);
											break;

											case 1:		// Name
												memcpypgm2ram(EP0_Data, (rom byte *)&ArtemStr, sizeof(ArtemStr));
												DataPointer = (rom byte *)&ArtemStr;
												DataCount = sizeof(ArtemStr);
											break;

											case 2:		// Product name
												memcpypgm2ram(EP0_Data, (rom byte *)&ProductStr, sizeof(ProductStr));
												DataPointer = (rom byte *)&ProductStr;
												DataCount = sizeof(ProductStr);
											break;

											case 3:		// Serial number
												memcpypgm2ram(EP0_Data, (rom byte *)&SerialStr, sizeof(SerialStr));
												DataPointer = (rom byte *)&SerialStr;
												DataCount = sizeof(SerialStr);
											break;

											case 4:		// Interface name
												memcpypgm2ram(EP0_Data, (rom byte *)&InterfaceStr, sizeof(InterfaceStr));
												DataPointer = (rom byte *)&InterfaceStr;
												DataCount = sizeof(InterfaceStr);
											break;

											default:
												DataCount = 1;	// 0 -- do not set to 0 here!
																// You'll see blue screen of death, then...
//									Need2Stop = 1;
												;
										}
									break;

									default:
										;
								}

							    EP0_In.Address = EP0_Data;
								if(DataCount < EP0_BufferSize)
									EP0_In.Count = DataCount;
								else
								    EP0_In.Count = EP0_BufferSize;
							    EP0_In.Status._byte = _USIE | _DAT1 | _DTSEN;
							break;

							default:
								;
						}
					break;

					case bmSET_xxxx:
						switch(SetupData.bRequest)
						{
							case bSET_ADDRESS:							// ADDRESSED
								RedLed(Off);		// 0x0011
								YellowLed(Off);
								GreenLed(On);
		
								USBAddress = SetupData.USBAddress;
								USBDeviceState = ADR_PENDING_STATE;
							break;

							case bSET_CONFIGURATION:					// CONFIGURED

						    	UEP1bits.EPHSHK = 1;
							   	UEP1bits.EPCONDIS = 1;
						    	UEP1bits.EPOUTEN = 1;
						    	UEP1bits.EPINEN = 1;

								EP1_In.Address = Report01._byte; // EP1_InData;
							    EP1_In.Count = EP1_BufferSize;
								EP1_In.Status._byte = _UCPU;

								USBDeviceState = CONFIGURED_STATE;
							break;

							default:
								;
						}

						DataCount = 0;
					    EP0_In.Address = EP0_Data;			
					    EP0_In.Count = 0;
					    EP0_In.Status._byte = _USIE | _DAT1 | _DTSEN;
					break;

					// Class descriptor
					case bmGET_INTERFACE:
						switch(SetupData.bRequest)
						{
							case bGET_DESCRIPTOR:
								switch(SetupData.DescriptorType)
								{
									case CLASS_HID:
										memcpypgm2ram(EP0_Data, (byte *)&HIDDescriptor1, EP0_BufferSize);
										DataPointer = (byte *)&HIDDescriptor1;
										DataCount = sizeof(HIDFormat1);
									break;

									case CLASS_REPORT:
										memcpypgm2ram(EP0_Data, ReportDescriptor_01, EP0_BufferSize);
										DataPointer = ReportDescriptor_01;
										DataCount = REPORT_SIZE_01;
									break;

									default:
										;
								}
						}
					    EP0_In.Address = EP0_Data;
						if(DataCount < EP0_BufferSize)
							EP0_In.Count = DataCount;
						else
						    EP0_In.Count = EP0_BufferSize;
					    EP0_In.Status._byte = _USIE | _DAT1 | _DTSEN;

					break;

					// Class (HID)
					case CLASS_HID:
						switch(SetupData.bRequest)
						{
							case bHID_SET_IDLE:		// Just start EP1 pipe here, not important to serve
								if(SetupData.wValue == 0)
								EP1_In.Status._byte = _USIE | _DAT0; // _UCPU | _DAT1;
							break;

							default:
								;
						}

						DataCount = 0;
					    EP0_In.Address = EP0_Data;
					    EP0_In.Count = 0;
					    EP0_In.Status._byte = _USIE | _DAT1 | _DTSEN;
					break;

					default:
						;
				}
				UCONbits.PKTDIS = 0;			//Enable transfer
			break;

			case OUT_TOKEN:
				RedLed(Off);		// 0x0100
				YellowLed(On);
				GreenLed(On);

				if(USBDeviceState >= ADDRESSED_STATE)
				{
					byte2log(0xCC);
				}

			    EP0_Out.Count = EP0_BufferSize;
			    EP0_Out.Address = (byte*)&SetupData;
			    EP0_Out.Status._byte = _USIE | _DAT1 | _DTSEN;
			break;

			default:
				;
		}
	}
	else if(USBStatus.Dir == IN)
	{
		switch(EP0_In.Status.PID)
		{
			case IN_TOKEN:
				RedLed(On);		// 0x0101
				YellowLed(Off);
				GreenLed(On);

				if(USBDeviceState >= ADDRESSED_STATE)
				{
					byte2log(0xBB);
//					if(DataCount > EP0_BufferSize)
//						ram2log(EP0_Data, EP0_BufferSize);
//					else
//						ram2log(EP0_Data, DataCount);
				}

				if(USBDeviceState == ADR_PENDING_STATE)
				{
					USBDeviceState = ADDRESSED_STATE;
					UADDR = USBAddress;

//					byte2log(0xAC);
//					byte2log(UADDR);
				}

				EP0_Out.Address = (byte*)&SetupData;
				EP0_Out.Count = EP0_BufferSize;
				EP0_Out.Status._byte = _USIE;

				EP0_In.Address = EP0_Data;

				DataPointer += EP0_BufferSize;
				DataCount = DataCount - EP0_BufferSize;

				if(DataCount > EP0_BufferSize)
				{
					memcpypgm2ram(EP0_Data, DataPointer, EP0_BufferSize);
					EP0_In.Count = EP0_BufferSize;
				}
				else
				{
					memcpypgm2ram(EP0_Data, DataPointer, DataCount);
					EP0_In.Count = DataCount;
				}

			    if(EP0_In.Status.DTS == 0)
					EP0_In.Status._byte = _USIE| _DAT1 | _DTSEN;
		        else
			        EP0_In.Status._byte = _USIE | _DAT0 | _DTSEN;

				if(Need2Stop)
					Stopped = 1;
			break;

			default:
				;
		}
	}
}


//
// --------------------------------------------------------------------------------------
void PollAndServeUSB(void)
{
	USBStatusReg USBStatus;
	static byte Logged = 0;

	if(Stopped && Logged)
		return;

	if(Stopped)
	{
		RedLed(On);		// 0x0111
		YellowLed(On);
		GreenLed(On);

		byte2log(0xFD);

		stream_ram2eeprom(Log, LogIndex);
		Logged = 1;

		return;
	}

	//
	if(UCONbits.USBEN == 0)
	{
	    UCON = 0;
    	UIE = 0;                                // Mask all USB interrupts
	    UCONbits.USBEN = 1;                     // Enable module & attach to bus
	    USBDeviceState = ATTACHED_STATE;
	}

	//
    if(USBDeviceState == ATTACHED_STATE)
    {
		if(!UCONbits.SE0)
		{
			UIR = 0;                        	// Clear all USB interrupts
			UIE = 0;                        	// Mask all USB interrupts
	        UIEbits.URSTIE = 1;             	// Unmask RESET interrupt
	        UIEbits.IDLEIE = 1;					// Unmask IDLE interrupt
	        USBDeviceState = POWERED_STATE;
	     }										// else wait until SE0 is cleared
	}

	//
	if(UIRbits.URSTIF && UIEbits.URSTIE)		// Reset
	{
		RedLed(Off);		// 0x0000
		YellowLed(Off);
		GreenLed(Off);

//		byte2log(0xFA);							// Decided not to log anything till addressed

    	UEIR = 0;                       		// Clear all USB error flags
    	UIR = 0;                        		// Clears all USB interrupts
    	UEIE = 0b10011111;              		// Unmask all USB error interrupts
    	UIE = 0b01111011;               		// Enable all interrupts except ACTVIE
    
    	UADDR = 0x00;                   		// Reset to default address
       
		UEP1=0x00;UEP2=0x00;UEP3=0x00;			// Reset all non-EP0 UEPn registers
    	UEP4=0x00;UEP5=0x00;UEP6=0x00;UEP7=0x00;
    	UEP8=0x00;UEP9=0x00;UEP10=0x00;UEP11=0x00;
    	UEP12=0x00;UEP13=0x00;UEP14=0x00;UEP15=0x00;

		UEP0 = 0x00;							// Init EP0 as a Ctrl EP
    	UEP0bits.EPHSHK = 1;
    	UEP0bits.EPCONDIS = 0;
    	UEP0bits.EPOUTEN = 1;
    	UEP0bits.EPINEN = 1;
 
    	while(UIRbits.TRNIF == 1)				// Flush any pending transactions
        	UIRbits.TRNIF = 0;

    	UCONbits.PKTDIS = 0;					// Make sure packet processing is enabled

	    EP0_Out.Count = EP0_BufferSize;
	    EP0_Out.Address = (byte*)&SetupData;
	    EP0_Out.Status._byte = _USIE | _DAT0 | _DTSEN;
	    EP0_In.Status._byte = _UCPU;

	    USBDeviceState = DEFAULT_STATE;
	}

	if(UIRbits.IDLEIF && UIEbits.IDLEIE)	// Idle
	{
		UIRbits.IDLEIF = 0;
	}

	if(UIRbits.SOFIF && UIEbits.SOFIE)		// SOF
	{
	    UIRbits.SOFIF = 0;
	}

    if(UIRbits.STALLIF && UIEbits.STALLIE)
	{
		UIRbits.STALLIF = 0;

		YellowLed(Blink);
		byte2log(0xFC);

		Stopped = 1;
	}

	if(UIRbits.UERRIF && UIEbits.UERRIE)	// Error
	{
		RedLed(Blink);

		byte2log(0xFB);
		byte2log(UEIR);

		Stopped = 1;
	}

    if(UIRbits.TRNIF && UIEbits.TRNIE)		// Transfer
    {
		USBStatus._byte = USTAT;

		switch(USBStatus.Endpoint)			// EP1 will never come here (?)
		{
			case 0:
				ServeEP0(USBStatus);
			break;

			case 1:
				ServeEP1(USBStatus);
			break;

			default:
				;
		}
		UIRbits.TRNIF = 0;
	}
}


//
// --------------------------------------------------------------------------------------
void main(void)
{
	InitLeds();
	InitIO();
	InitUSB();

	while(1)
	{
		DriveLeds();
		WorkIOut();
		PollAndServeUSB();			// SIE actually starts just here
	}
}


//	if(EP1_In.Status.UOWN)					// SIE owns memory
//		return;
