//
#ifndef AJO_H
#define AJO_H

// MCU
// --------------------------------------------------------------------------------------
#include <p18f2550.h>
#pragma list p = 18f2550


// Commons
// --------------------------------------------------------------------------------------
// Types
typedef unsigned char   byte;           // 8-bit
typedef unsigned int    word;           // 16-bit
typedef unsigned long   dword;          // 32-bit

typedef union
{
    word _word;
    struct
    {
        byte byte0;
        byte byte1;
    };
    struct
    {
        byte v[2];
    };
} WORD;
#define LSB(a)      ((a).v[0])
#define MSB(a)      ((a).v[1])


// USB status register
// --------------------------------------------------------------------------------------
#define OUT         0
#define IN          1

typedef union
{
	byte _byte;
	struct
	{
		unsigned :1;
		unsigned Odd:1;
		unsigned Dir:1;
		unsigned Endpoint:4;
		unsigned :1;
	};
} USBStatusReg;


// USB memory registers
// --------------------------------------------------------------------------------------
// Buffer Descriptor Status Register Initialization Parameters
#define _BSTALL     0x04				//Buffer Stall enable
#define _DTSEN      0x08				//Data Toggle Synch enable
#define _INCDIS     0x10				//Address increment disable
#define _KEN        0x20				//SIE keeps buff descriptors enable
#define _DAT0       0x00                //DATA0 packet expected next
#define _DAT1       0x40                //DATA1 packet expected next
#define _DTSMASK    0x40                //DTS Mask
#define _USIE       0x80                //SIE owns buffer
#define _UCPU       0x00                //CPU owns buffer

//
typedef union
{
	byte _byte;
    struct{
        unsigned BC8:1;
        unsigned BC9:1;
        unsigned BSTALL:1;              //Buffer Stall Enable
        unsigned DTSEN:1;               //Data Toggle Synch Enable
        unsigned INCDIS:1;              //Address Increment Disable
        unsigned KEN:1;                 //BD Keep Enable
        unsigned DTS:1;                 //Data Toggle Synch Value
        unsigned UOWN:1;                //USB Ownership
    };
    struct{
        unsigned :2;
        unsigned PID:4;                 //Packet Identifier
        unsigned :2;
    };
} BDStatusReg;                              //Buffer Descriptor Status Register

//
typedef union
{
    struct
    {
        BDStatusReg Status;
        byte Count;
        byte ADRL;						//Buffer Address Low
        byte ADRH;						//Buffer Address High
    };
    struct
    {
        unsigned :8;
        unsigned :8;
        byte* Address;					//Buffer Address
    };
} BDTable;								//Buffer Descriptor Table


// Endpoint 0 control transfer buffers
// --------------------------------------------------------------------------------------
#define EP0_BufferSize				8	// 16, 32, 64

// Types
typedef union
{
	struct
	{
		byte bmRequestType;
		byte bRequest;
		word wValue;
		word wIndex;
		word wLength;
	};

	struct
	{
		byte Foo;
		byte Foo1;
		byte bIndex;
		byte DescriptorType;
		byte Foo4;
		byte Foo5;
		byte DataSize;
	};
	struct
	{
		byte Foo;
		byte Foo2;
		byte USBAddress;
	};
}SetupDataFormat;

// USB PID: Token Types - See chapter 8 in the USB specification
// --------------------------------------------------------------------------------------
#define SETUP_TOKEN         0b00001101
#define OUT_TOKEN           0b00000001
#define IN_TOKEN            0b00001001


// USB bmRequestType
// --------------------------------------------------------------------------------------
#define bmGET_DESCRIPTOR			0b10000000		// 0x80
#define bmGET_CONFIGURATION			0b10000000		// 0x80
#define	bmGET_xxxx					0b10000000		// 0x80
#define bmGET_INTERFACE				0b10000001		// 0x81
#define bmSET_ADDRESS				0b00000000		// 0x00
#define bmSET_CONFIGURATION			0b00000000		// 0x00
#define	bmSET_xxxx					0b00000000		// 0x00

// USB bmRequest
// --------------------------------------------------------------------------------------
#define bGET_DESCRIPTOR				0x06

#define bSET_ADDRESS				0x05
#define bSET_CONFIGURATION			0x09


// USB Device States
// --------------------------------------------------------------------------------------
#define DETACHED_STATE          0
#define ATTACHED_STATE          1
#define POWERED_STATE           2
#define DEFAULT_STATE           3
#define ADR_PENDING_STATE       4
#define ADDRESSED_STATE         5
#define CONFIGURED_STATE        6


// USB descriptor types
// --------------------------------------------------------------------------------------
#define DEVICE						1
#define CONFIGURATION				2
#define STRING						3
#define INTERFACE					4
#define ENDPOINT					5
#define SOME01						6	// Something 2.0 related

// Class bmRequestType
// --------------------------------------------------------------------------------------
#define CLASS_HID					0x21
#define CLASS_REPORT				0x22
#define CLASS_PHISICAL				0x23

// HID bRequestType
// --------------------------------------------------------------------------------------
#define bHID_SET_IDLE				0x0A


// USB Device Descriptor Structure
// --------------------------------------------------------------------------------------
// Device descriptor
typedef struct
{
	byte bLength;
	byte bDscType;
	word bcdUSB;
	byte bDevCls;
	byte bDevSubCls;
	byte bDevProtocol;
	byte bMaxPktSize0;
	word idVendor;
	word idProduct;
	word bcdDevice;
	byte iMFR;
	byte iProduct;
	byte iSerialNum;
	byte bNumCfg;
} DeviceDescriptorFormat;

// USB Configuration Descriptor Structure
// --------------------------------------------------------------------------------------
// Configuration Attributes
#define _DEFAULT    0x01<<7         //Default Value (Bit 7 is set)
#define _SELF       0x01<<6         //Self-powered (Supports if set)
#define _RWU        0x01<<5         //Remote Wakeup (Supports if set)

//
typedef struct
{
	byte bLength;
	byte bDscType;
	word wTotalLength;
	byte bNumIntf;
	byte bCfgValue;
	byte iCfg;
	byte bmAttributes;
	byte bMaxPower;
} DeviceConfigurationFormat;

// USB Interface Descriptor Structure
// --------------------------------------------------------------------------------------
typedef struct
{
	byte bLength;
	byte bDscType;
	byte bInterfaceNum;
	byte bAltSettings;
	byte bNumEndpoints;
	byte bInterfaceClass;
	byte bInterfaceSubclass;
	byte bInterfaceProtocol;
	byte iInterface;
} DeviceInterfaceFormat;

// USB Endpoint Descriptor Structure
// --------------------------------------------------------------------------------------
typedef struct
{
	byte bLength;
	byte bDscType;
	byte bEndpointAdr;
	byte bmAttributes;
	word wMaxPacketSize;
	byte bInterval;
} EndPointFormat;

// HID
// --------------------------------------------------------------------------------------
typedef struct
{
	byte bLength;
	byte bDscType;
	word bcdHID;
	byte bCountryCode;
	byte bNumDsc;
	byte bDescType;
	word wDescSize;
} HIDFormat;

// HID (1 additional descriptor added)
typedef struct
{
	byte bLength;
	byte bDscType;
	word bcdHID;
	byte bCountryCode;
	byte bNumDsc;
	byte bDescType;
	word wDescSize;
	byte bDescType1;
	word wDescSize1;
} HIDFormat1;

// USB Configuration 01
// --------------------------------------------------------------------------------------
typedef struct
{
	DeviceConfigurationFormat		Config;
	DeviceInterfaceFormat			Interface;
	HIDFormat						Hid;
	EndPointFormat					EndPoint;
} USBConfigurationStruct;


// Report format 01
// --------------------------------------------------------------------------------------
#define EP1_BufferSize			4

//
typedef union
{
	byte _byte[EP1_BufferSize];

	struct
	{
		unsigned XL: 8;
		unsigned XH: 2;
		unsigned YL: 6;
		unsigned YH: 4;
		unsigned ZL: 4;
		unsigned ZH: 6;
		unsigned Buttons1: 2;
//		unsigned Buttons2: 8;
//		unsigned Buttons3: 4;
	};
} ReportFormat01;


//
extern volatile far ReportFormat01 Report01;


// USB functions
// --------------------------------------------------------------------------------------
extern void InitUSB(void);


// IO functions
// --------------------------------------------------------------------------------------
// ADC
#define ChannelsNum			3
extern WORD ADCSense[ChannelsNum];

extern void InitIO(void);
extern void WorkIOut(void);


// Auxiliary functions
// --------------------------------------------------------------------------------------
// Leds
#define BlinkDelay			32767
#define ShortBlinkDelay		16384

enum Modes {Off, On, Blink, Free};

extern void InitLeds(void);
extern void RedLed(unsigned char Mode);
extern void GreenLed(unsigned char Mode);
extern void YellowLed(unsigned char Mode);

extern void DriveLeds(void);

// Log
#define LOGSIZE					240
extern byte Log[128];
extern byte LogIndex;
extern byte Stopped;

extern void byte2log(byte data);
extern void ram2log(byte *data, byte size);

// ROM
extern void rom2rambytecpy(byte *dst, rom byte *src, byte size);

// EEPROM
extern void byte2eeprom(unsigned char data, unsigned char address);
extern void ram2eeprom(unsigned char eepdst, unsigned char *src, unsigned char size);
extern void stream_byte2eeprom(unsigned char data);
extern void stream_ram2eeprom(unsigned char *src, unsigned char size);


#endif // AJO_H
