//
#include "AJo.h"


// File variables
// --------------------------------------------------------------------------------------
static unsigned char RedLedState = Off;
static unsigned char GreenLedState = Off;
static unsigned char YellowLedState = Off;

// EEPROM stream count
static unsigned char eepstrm = 0x00;


// Leds
// --------------------------------------------------------------------------------------
void InitLeds(void)
{
	ADCON1 |= 0x0F;                 // Default all pins to digital

	TRISB = 0xFF;
	PORTB = 0x00;
	
	TRISBbits.TRISB2 = 0;			// Set pin RB2 to be output	
	TRISBbits.TRISB3 = 0;			// Set pin RB3 to be output
	TRISBbits.TRISB4 = 0;			// Set pin RB4 to be output
}

//
void RedLed(unsigned char Mode)
{
	RedLedState = Mode;
}

//
void GreenLed(unsigned char Mode)
{
	GreenLedState = Mode;
}

//
void YellowLed(unsigned char Mode)
{
	YellowLedState = Mode;
}

//
void DriveLeds(void)
{
	static unsigned short int RedCount = BlinkDelay;
	static unsigned short int GreenCount = BlinkDelay;
	static unsigned short int YellowCount = BlinkDelay;

	switch(RedLedState)
	{
		case Off:
			PORTBbits.RB4 = 0;
		break;

		case On:
			PORTBbits.RB4 = 1;
		break;

		case Blink:
			if(--RedCount == 0)
			{
				RedCount = BlinkDelay;
				PORTBbits.RB4 = ~LATBbits.LATB4;
			}
		break;

		case Free:
		default:
			;
	}

	switch(GreenLedState)
	{
		case Off:
			PORTBbits.RB2 = 0;
		break;

		case On:
			PORTBbits.RB2 = 1;
		break;

		case Blink:
			if(--GreenCount == 0)
			{
				GreenCount = BlinkDelay;
				PORTBbits.RB2 = ~LATBbits.LATB2;
			}
		break;

		case Free:
		default:
			;
	}

	switch(YellowLedState)
	{
		case Off:
			PORTBbits.RB3 = 0;
		break;

		case On:
			PORTBbits.RB3 = 1;
		break;

		case Blink:
			if(--YellowCount == 0)
			{
				YellowCount = BlinkDelay;
				PORTBbits.RB3 = ~LATBbits.LATB3;
			}
		break;

		case Free:
		default:
			;
	}
}


// ROM to RAM
// --------------------------------------------------------------------------------------
void rom2rambytecpy(byte *dst, rom byte *src, byte size)
{
	byte i = 0;
	for(; i < size; i++)
		*dst++ = *src++;
}


// Log
// --------------------------------------------------------------------------------------
void byte2log(byte data)
{
	Log[LogIndex++] = data;

	if(LogIndex >= LOGSIZE + 1)
		Stopped = 1;
}

//
void ram2log(byte *data, byte size)
{
	byte i = 0;
	for(; i < size; i++)
		byte2log(*data++);
}

// EEPROM
// --------------------------------------------------------------------------------------
void byte2eeprom(unsigned char data, unsigned char address)
{
	EECON1bits.EEPGD = 0;	/* WRITE step #1 */
	EECON1bits.WREN = 1;	/* WRITE step #2 */
	EEADR = address;		/* WRITE step #3 */
	EEDATA = data;			/* WRITE step #4 */
	EECON2 = 0x55;			/* WRITE step #5 */
	EECON2 = 0xAA;			/* WRITE step #6 */
	EECON1bits.WR = 1;		/* WRITE step #7 */
	while (!PIR2bits.EEIF)	/* WRITE step #8 */
		;
	PIR2bits.EEIF = 0;		/* WRITE step #9 */
}


//
void ram2eeprom(unsigned char eepdst, unsigned char *src,  unsigned char size)
{
	unsigned char i = 0;
	for(; i < size; i++)
		byte2eeprom(*src++, eepdst++);
}

//
void stream_byte2eeprom(unsigned char data)
{
	byte2eeprom(data, eepstrm++);
}

//
void stream_ram2eeprom(unsigned char *src, unsigned char size)
{
	unsigned char i = 0;
	for(; i < size; i++)
		byte2eeprom(*src++, eepstrm++);	
}
